# coding=utf-8

import logging

def setup_logger(name, level= logging.INFO):
    '''Set up a logger to print to terminal.
    Available levels: logging.(DEBUG|INFO|WARN|ERROR)'''
    logger = logging.getLogger(name)
    logger.setLevel(level)
    ch = logging.StreamHandler()
    formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
    ch.setFormatter(formatter)
    logger.addHandler(ch)
    return logger


# coding=utf-8

class NewdexAPIException(Exception):
    '''Exception class to handle general API Exceptions'''
    def __init__(self, response):
        self.code = ''
        self.message = 'Unknown Error'
        try:
            json_res = response
        except ValueError:
            print("Can't parse error response: {}".format(response.text))
            self.message = response.text
        else:
            if 'msg' in json_res:
                self.message = json_res['msg']
                self.code = json_res['code']

    def __str__(self):
        return 'NewdexAPIException {}: {}'.format(self.code, self.message)


# -*- coding: utf-8 -*-

import requests
import logging
import json
import time
import hashlib
import hmac
from newdex.util.logger import setup_logger
from newdex.util.exceptions import NewdexAPIException

class NewdexV1():
    def __init__(self, access_key, secret_key, symbol = 'theonlykarma-karma-wax', 
                    log_level = logging.INFO, max_retries = 3):
        self.key = access_key
        self.secret = secret_key
        self.host = 'https://api.newdex.io/v1'
        self.symbol = symbol
        self._retries = 0
        self.max_retries = max_retries

        self.logger = setup_logger('NewdexV1', log_level)
        self.logger.info('NewdexV1 initialized. Endpoint: %s, Symbol: %s, Max retries: %s.' % 
                                                (self.host, self.symbol, self.max_retries))

    def open_orders(self):
        '''Get all open orders from symbol pair.'''
        return self._curl_newdex('/order/orders?', state= 'pending')

    def filled_orders(self):
        '''Get all open orders from symbol pair.'''
        return self._curl_newdex('/order/orders?', state= 'filled')

    def canceled_orders(self):
        '''Get all open orders from symbol pair.'''
        return self._curl_newdex('/order/orders?', state= 'canceled')

    def order_history(self):
        '''Get all open orders from symbol pair.'''
        return self._curl_newdex('/order/orders?', state= 'history')

    def cancel_order(self, trx_id):
        '''Cancel order based on trx_id.'''
        return self._curl_newdex('/order/cancel?', cancel= trx_id)

    def get_pairs(self):
        '''Get the list of all exchange pairs.'''
        return self._curl_newdex('/common/symbols')

    def get_tickers(self):
        '''Get the market of all exchange pairs.'''
        return self._curl_newdex('/tickers')

    def get_ticker(self):
        '''Get the market of a particular exchange pair.'''
        return self._curl_newdex('/ticker?symbol=')

    def get_price(self):
        '''Get the price of an exchange pair.'''
        return self._curl_newdex('/price?symbol=')

    def get_depth(self):
        '''Get order book (quoted depth).'''
        return self._curl_newdex('/depth?symbol=')

    def get_trades(self):
        '''Get deals record of exchange pair.'''
        return self._curl_newdex('/trades?symbol=')

    def get_candles(self):
        '''Get K-Line data of exchange pair.'''
        return self._curl_newdex('/candles?symbol=')

    def _curl_newdex(self, path, state = False, cancel = False):
        '''Build Newdex request url and auto retry on failure.'''
        if state:
            message = 'api_key=' + self.key + '&size=50&state=' + state + '&symbol=' + self.symbol + '&timestamp=' + str(int(time.time()))
            url = self.host + path + message + "&sign=" + self._sign(message)
        elif cancel:
            message = 'api_key=' + self.key + '&trx_id=' + cancel + '&timestamp=' + str(int(time.time()))
            url = self.host + path + message + "&sign=" + self._sign(message)
        else:
            url = self.host + path + self.symbol if '?' in path else self.host + path

        self.logger.debug('Sending request to %s' % url)
        response = requests.get(url).json()

        if response["code"] != 200:
            self._retries += 1
            if self._retries >= self.max_retries:
                self.logger.error('Max retries with error code %s, exiting.' % response['code'])
                raise NewdexAPIException(response)
            else:
                self.logger.warning('Request returned with error code %s, retrying...' % response['code'])
                self._curl_newdex(path, state, cancel)
        self._retries = 0
        return response["data"]

    def _sign(self, message):
        '''Generate HmacSHA256 signature according to message.'''
        return hmac.new(key=bytes(self.secret, 'UTF-8'), msg=bytes(message, 'UTF-8'), 
        digestmod=hashlib.sha256).hexdigest()


#!/usr/bin/env python
from setuptools import setup, find_packages
from os.path import join, dirname

here = dirname(__file__)

setup(name='newdex',
      version='0.1.0',
      description="Unnoficial Newdex API wrapper for Python 3.",
      long_description=open(join(here, 'README.md')).read(),
      long_description_content_type="text/markdown",
      author='canokaue',
      author_email='kaue.cano@quan.digital',
      license='MIT',
      url='https://github.com/quan-digital/newdex',
      download_url = 'https://github.com/quan-digital/newdex/dist/newdex-0.1.0.tar.gz',
      install_requires=[
        'requests==2.23.0'
      ],
      packages=find_packages(),
      keywords = ['newdex', 'cryptocurrency', 'exchange', 'crypto-api', 'crypto-exchange', 'descentralized-exchange', 
      'newdex-api', 'eosio', 'api-wrapper'],
      classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Financial and Insurance Industry',
        'Topic :: Software Development',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Operating System :: OS Independent',
      ],
      )
